import java.awt.Canvas;
import java.awt.Color;
import java.awt.EventQueue;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import java.awt.Toolkit;


/*                                Angle splines                                 */
/*                             www.mosismath.com                                */

public class mainFrame {

	private JFrame frmAngleSplines;
	private JPanel pGraph; 

	int order = 6; // number of reference points
    int datapoints = 1000; // number of datapoints for the interpolation
   
    double integral = 0;
    public double deltaX = 0.5;
    double alpha = 45;
    double beta = -20;

    double[] yp = new double[datapoints];
    double[] xp = new double[datapoints];
    double[] y_k = new double[order];
    double[] x_k = new double[order];
    double[] a = new double[order];
    double[] b = new double[order];
    double[] c = new double[order];
    double[] d = new double[order];
    double[] h = new double[order];
    public double[][] m = new double[order][order];
    public double[] y = new double[order];
    public double[] x = new double[order];
    private JTextField[] tbs = new JTextField[order];
    private JTextField[] tbx = new JTextField[order];
    private JTextField textBoxY1;
    private JTextField textBoxY2;
    private JTextField textBoxY3;
    private JTextField textBoxY4;
    private JTextField textBoxY5;
    private JTextField textBoxY6;
    private JTextField textBoxX1;
    private JTextField textBoxX2;
    private JTextField textBoxX3;
    private JTextField textBoxX4;
    private JTextField textBoxX5;
    private JTextField textBoxX6;
    
    public class TPoint  {
		public float X;
        public float Y;	
	}

    public class myCanvas extends Canvas {
    	
		private static final long serialVersionUID = 1L;
		
		public void paint(Graphics g) { 		 			              
			  drawGraph(g, datapoints, true);
		}
    }
   
    public boolean CalcParameters()
    {
    	 int i, k;
         for (i = 0; i < order; i++)  // calculate a parameters
         {
             a[i] = y_k[i];
         }
         for (i = 0; i < order - 1; i++)
         {
             h[i] = x_k[i + 1] - x_k[i];
         }

         // empty matrix for c parameters
         for (i = 0; i < order; i++)
         {
             for (k = 0; k < order; k++)
             {
                 m[i][k] = 0.0;
                 y[i] = 0.0;
                 x[i] = 0.0;
             }
         }

         // fill matrix for c parameters        
         m[0][0] = 2.0 * h[0];
         m[0][1] = h[0];
         y[0] = 3 *((a[1] - a[0]) / h[0] - Math.tan(alpha * Math.PI / 180));

         for (i = 0; i < order - 2; i++)
         {
             m[i + 1][i] = h[i];
             m[i + 1][i + 1] = 2.0 * (h[i] + h[i + 1]);
             if (i < order - 2)
                 m[i + 1][i + 2] = h[i + 1];

             if ((h[i] != 0.0) && (h[i + 1] != 0.0))
                 y[i+1] = ((a[i + 2] - a[i + 1]) / h[i + 1] - (a[i + 1] - a[i]) / h[i]) * 3.0;
             else
                 y[i+1] = 0.0;
         }

         m[order - 1][order - 2] = h[order - 2];
         m[order - 1][order - 1] = 2.0 * h[order - 2];
         y[order - 1] = 3.0 * (Math.tan(beta * Math.PI / 180) - (a[order - 1] - a[order - 2]) / h[order - 2]);

         TGauss gauss = new TGauss(order, m, y);
         if (gauss.Eliminate())
         {
             x=gauss.Solve();
            
             for (i = 0; i < order; i++)
             {
                 c[i] = x[i];
             }
             for (i = 0; i < order ; i++) // calculate b and d parameters
             {
                 if (h[i] != 0.0)
                 {
                     d[i] = 1.0 / 3.0 / h[i] * (c[i + 1] - c[i]);
                     b[i] = 1.0 / h[i] * (a[i + 1] - a[i]) - h[i] / 3.0 * (c[i + 1] + 2 * c[i]);
                 }
             }
             return true;
         }
         else
             return false;
    }

     

    public void Interpolate(int order, int resolution)
    {
        int i, k;
        double timestamp;
        for (i = 0; i < order - 1; i++)
        {
            for (k = 0; k < resolution; k++)
            {
                timestamp = (double)(k) / (double)(resolution) * h[i];
                yp[i * resolution + k] = a[i] + b[i] * (timestamp) + c[i] * Math.pow(timestamp, 2) + d[i] * Math.pow(timestamp, 3);
                xp[i * resolution + k] = timestamp + x_k[i];
            }
        }
    }

   private void calculate()
   {
	   int i;
       boolean isError = false;
       
      for (i = 0; i < order; i++)
      {
          if (tbs[i].getText() != "")
          {
        	  y_k[i] = Double.parseDouble(tbs[i].getText());
              x_k[i] = Double.parseDouble(tbx[i].getText());
          }
          else
          {
              y_k[i] = 0.0;
              x_k[i] = 0.0;
          }
          if (i > 0)
          {
               if (x_k[i] < x_k[i - 1])
                  isError = true;
          }
      }
      if (isError)
         JOptionPane.showMessageDialog(frmAngleSplines, "Time stamps must increase");
                     
      if (CalcParameters())
      {
           Interpolate(order, datapoints / order);
                 
      }
      else
    	  JOptionPane.showMessageDialog(frmAngleSplines,"Matrix calculattion");      
   }

   
   public void drawGraph(Graphics g, int maxPoints, boolean doClear)
   {
       TPoint p1, p2;
       int j;
       double maxTime = -1.0;
       double maxVal = -1.0;
       double minVal = 1.0;
       double scalefactor;
       boolean drawCeroline;
       for (j = 0; j < maxPoints - 1; j++)
       {
          if (maxTime < xp[j])
        	  maxTime = xp[j];
          if (maxVal < yp[j])
        	  maxVal = yp[j];
          if (minVal > yp[j])
        	  minVal = yp[j];        
       }
       maxTime = maxTime * 1.05;
       maxTime = maxTime * 1.05;
       if (minVal < 0)
       {
           if (maxVal > 0)
           {
               if (maxVal > Math.abs(minVal))
                   scalefactor = pGraph.getHeight() / maxVal / 2.2;
               else
                   scalefactor = pGraph.getHeight() / Math.abs(minVal) / 2.2;
           }
           else
               scalefactor = 5.0;
           drawCeroline = true;
       }
       else
       {
           scalefactor = pGraph.getHeight() / (maxVal) / 1.1;
           drawCeroline = false;
       }
       p1 = new TPoint();
       p2 = new TPoint();
       if (doClear)
       	g.clearRect(0, 0, pGraph.getWidth(), pGraph.getHeight());
      
       g.setColor(new Color(0, 0, 255));
       if (drawCeroline)
       {
           p1.X = 0;
           p1.Y = (pGraph.getHeight() / 2);
           p2.X = pGraph.getWidth();
           p2.Y = (pGraph.getHeight() / 2);
           g.setColor(new Color(0, 0, 0));
           g.drawLine((int)Math.round(p1.X),(int)Math.round(p1.Y),(int)Math.round( p2.X),(int)Math.round(p2.Y)); 
       }
       g.setColor(new Color(0, 0, 255));
       if (maxTime > 0) // draw interpolated graph
       {
           for (j = 0; j < maxPoints - 1; j++)
           {     
               p1.X = (int)Math.round(xp[j] * pGraph.getWidth() / (double)maxTime);
               p2.X = (int)Math.round(xp[j + 1] * pGraph.getWidth() / (double)maxTime);
               if (drawCeroline)
               {
                   p1.Y = (pGraph.getHeight() / 2) - (int)Math.round(yp[j] * scalefactor);
                   p2.Y = (pGraph.getHeight() / 2) - (int)Math.round(yp[j + 1] * scalefactor);
               }
               else
               {
                   p1.Y = (pGraph.getHeight() - (int)Math.round(yp[j] * scalefactor));
                   p2.Y = (pGraph.getHeight() - (int)Math.round(yp[j + 1] * scalefactor));
               }
               if (p2.X > p1.X)
              	   g.drawLine((int)Math.round(p1.X),(int)Math.round(p1.Y),(int)Math.round( p2.X),(int)Math.round(p2.Y)); 
           }
           g.setColor(new Color(255, 0, 0));
           for (j = 0; j < order; j++) // mark reference points
           {
        	   p1.X = (int)Math.round(x_k[j] * pGraph.getWidth() / (double)maxTime);
               if (drawCeroline)
                   p1.Y = (pGraph.getHeight() / 2) - (int)Math.round(y_k[j] * scalefactor);
               else
                   p1.Y = (pGraph.getHeight() - (int)Math.round(y_k[j] * scalefactor));
               g.drawOval((int)Math.round(p1.X-2), (int)Math.round(p1.Y-2), 4, 4);
           }
       }
   }
	
	
	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					mainFrame window = new mainFrame();
					window.frmAngleSplines.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	/**
	 * Create the application.
	 */
	public mainFrame() {
		int i;
		initialize();
		tbs[0] = textBoxY1;
		tbs[1] = textBoxY2;
		tbs[2] = textBoxY3;
		tbs[3] = textBoxY4;
		tbs[4] = textBoxY5;
		tbs[5] = textBoxY6;
		tbx[0] = textBoxX1;
		tbx[1] = textBoxX2;
		tbx[2] = textBoxX3;
		tbx[3] = textBoxX4;
		tbx[4] = textBoxX5;
		tbx[5] = textBoxX6;
		
		// init some sample y values
        y_k[0] = 0.5;
        y_k[1] = 3.7;
        y_k[2] = 3.8;
        y_k[3] = 4.0;
        y_k[4] = 2.2;
        y_k[5] = 0.2;

        // timestamp to y value
        for (i = 1; i <= order; i++)
        {
            x_k[i - 1] = deltaX * (double)i;
        }
       
       for (i = 0; i < order; i++)
       {
    	
           tbs[i].setText(Double.toString(y_k[i]));
           tbx[i].setText(Double.toString(x_k[i]));

       }
	}

	/**
	 * Initialize the contents of the frame.
	 */
	private void initialize() {
		frmAngleSplines = new JFrame();
		frmAngleSplines.setTitle("Angle splines");
		frmAngleSplines.setIconImage(Toolkit.getDefaultToolkit().getImage(mainFrame.class.getResource("/Ressource/Icon.png")));
		frmAngleSplines.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frmAngleSplines.setBounds(100, 100, 840, 536);
		frmAngleSplines.getContentPane().setLayout(null);
		pGraph = new JPanel();
		final Canvas canvas = new myCanvas();
		canvas.setBackground(Color.WHITE);
		pGraph.addComponentListener(new ComponentAdapter() {
			@Override
			public void componentResized(ComponentEvent arg0) {
				canvas.setBounds(0, 0, pGraph.getWidth(), pGraph.getHeight());
			}
		});
		pGraph.setBackground(Color.WHITE);
		pGraph.setBounds(10, 11, 800, 393);
		frmAngleSplines.getContentPane().add(pGraph);
		pGraph.add(canvas);
		pGraph.setLayout(null);
		
		textBoxY1 = new JTextField();
		textBoxY1.setColumns(10);
		textBoxY1.setBounds(49, 416, 62, 30);
		frmAngleSplines.getContentPane().add(textBoxY1);
		
		textBoxY2 = new JTextField();
		textBoxY2.setColumns(10);
		textBoxY2.setBounds(121, 416, 62, 30);
		frmAngleSplines.getContentPane().add(textBoxY2);
		
		textBoxY3 = new JTextField();
		textBoxY3.setColumns(10);
		textBoxY3.setBounds(193, 416, 62, 30);
		frmAngleSplines.getContentPane().add(textBoxY3);
		
		textBoxY4 = new JTextField();
		textBoxY4.setColumns(10);
		textBoxY4.setBounds(265, 416, 62, 30);
		frmAngleSplines.getContentPane().add(textBoxY4);
		
		textBoxY5 = new JTextField();
		textBoxY5.setColumns(10);
		textBoxY5.setBounds(337, 416, 62, 30);
		frmAngleSplines.getContentPane().add(textBoxY5);
		
		textBoxY6 = new JTextField();
		textBoxY6.setColumns(10);
		textBoxY6.setBounds(409, 416, 62, 30);
		frmAngleSplines.getContentPane().add(textBoxY6);
		
		textBoxX1 = new JTextField();
		textBoxX1.setColumns(10);
		textBoxX1.setBounds(49, 457, 62, 30);
		frmAngleSplines.getContentPane().add(textBoxX1);
		
		textBoxX2 = new JTextField();
		textBoxX2.setColumns(10);
		textBoxX2.setBounds(121, 457, 62, 30);
		frmAngleSplines.getContentPane().add(textBoxX2);
		
		textBoxX3 = new JTextField();
		textBoxX3.setColumns(10);
		textBoxX3.setBounds(193, 457, 62, 30);
		frmAngleSplines.getContentPane().add(textBoxX3);
		
		textBoxX4 = new JTextField();
		textBoxX4.setColumns(10);
		textBoxX4.setBounds(265, 457, 62, 30);
		frmAngleSplines.getContentPane().add(textBoxX4);
		
		textBoxX5 = new JTextField();
		textBoxX5.setColumns(10);
		textBoxX5.setBounds(337, 457, 62, 30);
		frmAngleSplines.getContentPane().add(textBoxX5);
		
		textBoxX6 = new JTextField();
		textBoxX6.setColumns(10);
		textBoxX6.setBounds(409, 457, 62, 30);
		frmAngleSplines.getContentPane().add(textBoxX6);
		
		JLabel lblNewLabel = new JLabel("Y");
		lblNewLabel.setFont(new Font("Tahoma", Font.PLAIN, 14));
		lblNewLabel.setBounds(20, 423, 46, 14);
		frmAngleSplines.getContentPane().add(lblNewLabel);
		
		JLabel lblX = new JLabel("X");
		lblX.setFont(new Font("Tahoma", Font.PLAIN, 14));
		lblX.setBounds(20, 464, 46, 14);
		frmAngleSplines.getContentPane().add(lblX);
		
		JButton btCalc = new JButton("Calculate");
		btCalc.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent e) {
				calculate();
				canvas.repaint();
			}
		});
		btCalc.setBounds(698, 415, 112, 72);
		frmAngleSplines.getContentPane().add(btCalc);
	}

}
